#include <qapp.h>
#include <qscrbar.h>
#include <qcolor.h>
#include <qgrpbox.h>
#include <qpainter.h>
#include <qlayout.h>
#include <qregexp.h> 

#include "color.h"

// Konstruktor fr farbiges Rechteck, 
// anfangs wei.
ColorWidget::ColorWidget(QWidget *parent, const char *name )
        : QWidget( parent, name ) {
    r = b = g = 255;
}

// Slots fr Einstellung der Farbkomponenten
void ColorWidget::set_red(int red) {
  ColorWidget::setColor(red, g, b);
}

void ColorWidget::set_blue(int blue) {
  ColorWidget::setColor(r, g, blue);
}

void ColorWidget::set_green(int green) {
  ColorWidget::setColor(r,green, b);
}

//Private Methode zum Setzen aller drei Farbwerte
//Lst 'repaint()' aus, wenn sich die Farbe gendert hat.
void ColorWidget::setColor(int red, int green, int blue) {
  if (red < 0)
    red = 0;
  if (green < 0)
    green = 0;
  if (blue < 0)
    blue = 0;
  if (red > 255)
    red = 255;
  if (green > 255)
    green = 255;
  if (blue > 255)
    blue = 255;
  if (r == red && b == blue && g == green)
    return;
  r = red;
  b = blue;
  g = green;
  repaint(FALSE);
}

// Von 'repaint()' ausgelstes Ereignis,
// fllt Rechteck mit der aktuellen Farbe
void ColorWidget::paintEvent( QPaintEvent *e) {
  QPainter p;
  const QColor c(r,g,b);

  p.begin(this);
  p.setBrush(c);
  p.drawRect(0,0,width(),height());
  p.end();
}

void ColorWidget::paintColor(QPainter *p) {
  const QColor c(r,g,b);
  p->setBrush(c);
  p->drawRect(0,0,width(),height());
}  

// Klasse LineEditInt: Zuerst statische Klassenvariable
//                     fr ganzzahligen reg. Ausdruck definieren
//                     dann Konstruktoren
QRegExp LineEditInt::intRegexp = QRegExp("^[0-9]+$");

LineEditInt::LineEditInt(QWidget *parent, const char *name) :
  QLineEdit(parent, name) {
  r = NULL;
  init(0);
}

LineEditInt::LineEditInt(QRangeControl range,
			 QWidget *parent, const char *name) :
  QLineEdit(parent, name) {
  r = new QRangeControl(range.minValue(), range.maxValue(),
			range.lineStep(), range.pageStep(),
			range.value());
  init(r->value());
}

// Destruktor: Wenn QRangeControl existiert, freigeben
LineEditInt::~LineEditInt() {
  if (r)
    delete r;
}

// Gemeinsame Mini-Initialisierung aller LineEditInt-Konstruktoren:
// nderungen des Textfeldes mit 'setNum()' verbinden.
void LineEditInt::init(int v) {
  connect(this, SIGNAL(textChanged(const char *)), this, 
	  SLOT(setNum(const char *)));
  setNum(v);
}

// setNum: Aktualisiert ggfs. die lokale Kopie des Werts
// und schickt 'valueChanged'-Signal
void LineEditInt::setNum(int val) {
  int tmp;
  if (val == value) 
    return;
  if (r) {
    r->setValue(val);
    tmp = r->value();
  } else {
    tmp = val;
  }
  if (tmp == value)
    return;
  value = tmp;
  QString s;
  s.setNum(value);
  setText(s);
  emit valueChanged(value);
}

// 'Slot' setNum(): Akzeptiert den String nur,
// wenn er leer ist oder auf den ganzzahligen reg. Ausdruck
// pat. Enthlt der String ein falsches Zeichen, setzt
// die Methode das Textfeld auf den letzten Wert zurck
void LineEditInt::setNum(const char *s) {
  char str[20];
  if (strlen(s) == 0)
    return;
  if (intRegexp.match(s) == -1) {
    sprintf(str,"%1d",value);
    setText(str);
  } else
    setNum(atoi(s));
}


// Konstruktor: Nur ein Aufruf des Basisklassen-Konstruktors 
MyScrollBar::MyScrollBar(int min, int max, int stepValue,
 			 int stepPage, int value, 
 			 Orientation or, QWidget *parent,
 			 const char *name) :
   QScrollBar(min, max, stepValue, stepPage, value, or, parent, name) { 
}

// Slot 'setValue': Benutzt setValue() der Basisklasse nur, wenn sich
// der Wert gendert hat. 

void MyScrollBar::setValue(int v) {
  if (v == value())
    return;
  QScrollBar::setValue(v);
}

    

int main( int argc, char **argv )
{
  static char *colorNames[3] = { "red", "green", "blue" };
  static MyScrollBar *sb[3];
  static LineEditInt *label[3];
  register int i;

  QApplication a( argc, argv );
  
  QGroupBox *box0 = new QGroupBox;
  box0->setFrameStyle( QFrame::Panel | QFrame::Sunken);
  QBoxLayout *b0_layout = new QBoxLayout( box0, QBoxLayout::LeftToRight,
					  10);
  for (i = 0; i < 3; i++) {
    QBoxLayout *s_layout = new QBoxLayout(QBoxLayout::BottomToTop);
    b0_layout->addLayout(s_layout,0);
    // Scrollbar und Textfeld fr ganze Zahlen erzeugen.
    sb[i]= new MyScrollBar(0,255,1,10,255,
			  QScrollBar::Vertical,box0, colorNames[i]);
    label[i] = new LineEditInt(QRangeControl(0,255,1,10,255),box0);
    // Farbe fr Textfeld einstellen (rot/grn/blau)
    QColorGroup cg = label[i]->colorGroup();
    QColorGroup new_cg(cg.foreground(), cg.background(),
		       cg.light(), cg.dark(), cg.mid(),
		       QColor(colorNames[i]), cg.base());
    label[i]->setPalette(QPalette(new_cg, new_cg, new_cg));
    // Im Textfeld den aktuellen Wert des Scrollbars anzeigen,
    // die maximale Textbreite setzen
    label[i]->setNum(sb[i]->value());
    label[i]->setMaxLength(3);
    label[i]->adjustSize();
    // Gren fr Textfeld und Scrollbar setzen und beide dem
    // Layout zur Verwaltung berreichen
    label[i]->setMaximumSize(label[i]->size());
    int w = label[i]->fontMetrics().width("999999");
    label[i]->setMinimumSize(w, label[i]->height());
    sb[i]->setMinimumSize(16,125);
    sb[i]->setMaximumSize(w+5,255); 
    s_layout->addWidget(sb[i]);
    s_layout->addWidget(label[i]);
  }
  // Rechteck fr Farbanzeige erzeugen und dem Layout berreichen
  ColorWidget cw(box0);
  cw.setMinimumSize(125,125); // Mindestens so gro wie Scrollbar
  b0_layout->addWidget(&cw,8);

  // Scrollbars und Textfelder per 'valueChanged'-Signal
  // miteinander verbinden
  for (i = 0; i < 3; i++) {
    sb[i]->connect(sb[i], SIGNAL(valueChanged(int)), 
		   label[i], SLOT(setNum(int)));
    label[i]->connect(label[i], SIGNAL(valueChanged(int)),
		      sb[i], SLOT(setValue(int)));
  }
  // Scrollbars mit dem Rechteck verbinden. 
  cw.connect(sb[0], SIGNAL(valueChanged(int)), &cw, SLOT(set_red(int)));
  cw.connect(sb[1], SIGNAL(valueChanged(int)), &cw, SLOT(set_green(int)));
  cw.connect(sb[2], SIGNAL(valueChanged(int)), &cw, SLOT(set_blue(int)));
  // Applikation sichtbar machen und 
  // Kontrolle an sie bergeben
  b0_layout->activate();
  box0->show();
  return a.exec();
}
