#include <qwindow.h>
#include <qpainter.h>
#include <qscrbar.h>
#include <qpntarry.h>
#include <qrect.h>
#include <qapp.h>
#include <math.h>

class House : public QWidget
{
  Q_OBJECT;
  public: 
    House();
    void paintEvent(QPaintEvent *);
    void resizeEvent(QResizeEvent *);
  public slots:
    void scale(double factor);
  signals:
    void sendResize(QWidget *, QResizeEvent *);
  private:
    double size;
};

House::House() {
  size = 1.0;
  setBackgroundColor(white);
}

void House::scale(double factor) {
  if (factor > 0.0 && factor != size)
    size = factor;
  update();
}

void House::resizeEvent(QResizeEvent *e) {
  emit sendResize(this, e);
}

void House::paintEvent(QPaintEvent *) 
{
  QPainter p;
  QWMatrix matrix;
  QPointArray pts, r;
  register int i;
  pts.setPoints(6, -20,-30,  20,-30,  20,10,  0,30, 
		   -20,10,  -20,-30);
  matrix.translate(width()  / 2, 
		   height() / 2);
  matrix.scale(size, -size);
  p.begin(this);
  p.setWorldMatrix(matrix);
  p.drawPolyline(pts);
  p.end();
}

class MyScrollBar: public QScrollBar
{
  Q_OBJECT;
  public: 
    MyScrollBar(QWidget *parent, const char *name = 0);
  public slots:
    void floatValue(int);
    void handleResize(QWidget *parent, QResizeEvent *e);
  signals:
    void newValue(double);
};

MyScrollBar::MyScrollBar(QWidget *parent, const char *name) 
  : QScrollBar(parent, name)
{
  setOrientation(QScrollBar::Horizontal);
  move(0, parent->height() - 16);
  resize(parent->width() , 16);
  setValue((maxValue() + minValue()) / 2);
  connect(this,SIGNAL(valueChanged(int)), SLOT(floatValue(int)));
}
  
void MyScrollBar::floatValue( int v) 
{
  double c;
  int half = (maxValue() + minValue()) / 2;
  c = (double)v/half;
  emit newValue(c);
}

void MyScrollBar::handleResize(QWidget *parent, QResizeEvent *e) {
  resize(parent->width(), 16);
  move(0, parent->height() -16);
}

#include "viewp.moc"

int main(int argc, char **argv) {
  QApplication app(argc, argv);
  House h;
  h.resize(300,300);
  h.setMaximumSize(600,600);
  h.setMinimumSize(120,160);
  MyScrollBar s(&h);
  s.connect(&s,SIGNAL(newValue(double)), &h, SLOT(scale(double)));
  h.connect(&h,SIGNAL(sendResize(QWidget *, QResizeEvent *)), 
	    &s, SLOT(handleResize(QWidget *, QResizeEvent *)));
  app.setMainWidget(&h);
  h.show();
  return app.exec();
}
